package boot.common.utils;

import cn.hutool.core.date.DateUtil;
import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTVerificationException;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
public class JwtToken {

    private static String jwtKey;
    private static Integer expiredTimeIn;
    private static Integer defaultScope = 8;

    @Value("${school.security.jwt-key}")
    public void setJwtKey(String jwtKey) {
        JwtToken.jwtKey = jwtKey;
    }

    @Value("${school.security.token-expired-in}")
    public void setExpiredTimeIn(Integer expiredTimeIn) {
        JwtToken.expiredTimeIn = expiredTimeIn;
    }

    public static Optional<Map<String, Claim>> getClaims(String token) {
        //存储解码后的JWT
        DecodedJWT decodedJWT;
        //创建一个算法对象algorithm，使用HMAC 256加密算法和JwtToken.jwtKey作为密钥
        Algorithm algorithm = Algorithm.HMAC256(JwtToken.jwtKey);
        //JWT.require(algorithm)指定了需要使用的算法（在本例中是HMAC 256）。这告诉JWTVerifier对象应该使用哪种算法来验证JWT。
        //.build()方法构建并返回一个JWTVerifier对象，该对象准备好用于验证JWT。
        JWTVerifier jwtVerifier = JWT.require(algorithm).build();
        try {
            //使用jwtVerifier验证输入的token，并将解码后的JWT存储在decodedJWT变量中。
            decodedJWT = jwtVerifier.verify(token);
        } catch (JWTVerificationException e) {
            return Optional.empty();
        }
        return Optional.of(decodedJWT.getClaims());
    }

    public static Boolean verifyToken(String token) {
        try {
            Algorithm algorithm = Algorithm.HMAC256(JwtToken.jwtKey);
            JWTVerifier verifier = JWT.require(algorithm).build();
            verifier.verify(token);
        } catch (JWTVerificationException e) {
            return false;
        }
        return true;
    }

    public static String getExpireTime(String token) {
        return DateUtil.formatDateTime(JWT.decode(token).getExpiresAt());
    }


    public static String makeToken(Long uid, String uName, Integer scope) {
        return JwtToken.getToken(uid, uName, scope);
    }

    public static String makeToken(Long uid, String uName) {
        return JwtToken.getToken(uid, uName, JwtToken.defaultScope);
    }

    private static String getToken(Long uid, String uName, Integer scope) {
        Algorithm algorithm = Algorithm.HMAC256(JwtToken.jwtKey);
        Map<String, Date> map = JwtToken.calculateExpiredIssues();

        return JWT.create()
                .withClaim("uid", uid)
                .withClaim("scope", scope)
                .withClaim("uName", uName)
                .withExpiresAt(map.get("expiredTime"))
                .withIssuedAt(map.get("now"))
                .sign(algorithm);
    }

    private static Map<String, Date> calculateExpiredIssues() {
        Map<String, Date> map = new HashMap<>();
        Calendar calendar = Calendar.getInstance();
        Date now = calendar.getTime();
        calendar.add(Calendar.SECOND, JwtToken.expiredTimeIn);
        map.put("now", now);
        map.put("expiredTime", calendar.getTime());
        return map;
    }
}
